/*
 . 
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h> /*F_CPU = 4000000; _delay_ms max = 65,535ms; _delay_us max = 192us */


#include "tr2.h"

/*
4000000 meg  RC
*/


ISR(TIMER2_OVF_vect){
	if ( --time[_led] == 0 )	time_flag|= BIT(_led);
	if ( --time[_01sek] == 0 )	time_flag|= BIT(_01sek);
}

void avr_init(void)
{
	TCCR2=BIT(CS20);
	TIMSK=BIT(TOIE2);//BIT(TOIE0)|
	
	DDRDRIVE|=DRIVE_TEN_1|DRIVE_TEN_2;
	PORTDRIVE|=swt_t2;
	DDRZN|=BITZN;
	DDRLED=0xFF;

	sei();

    return;
}
//-------------------EEPROM-------------------------
unsigned char EEPROM_read(unsigned int uiAddress)
{
	while(EECR & (1<<EEWE));
	EEAR = uiAddress;
	EECR |= (1<<EERE);
	return EEDR;
}

void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	if ( EEPROM_read(uiAddress)!=ucData){
		while(EECR & (1<<EEWE));
		cli();
		EEAR = uiAddress;
		EEDR = ucData;
		EECR |= (1<<EEMWE);
		EECR |= (1<<EEWE);
		sei();
		}
}
//-------------END------EEPROM----------------------


//**********1wire************
void pullup_off(void){
	DDR_1wire&=~pwire0;
	DDR_1wire&=~pwire1;
	PORT_1wire&=~pwire0;
	PORT_1wire&=~pwire1;
}

unsigned char readbit(void){
unsigned char i;
	cli();
	DDR_1wire|=out_1wire;//line down
	
	asm("nop");
	asm("nop");
	asm("nop");//1.25us
	DDR_1wire&=~out_1wire;//line up through 1.25us
	_delay_us(11);
	i=PIN_1wire;
	sei();
	_delay_us(47);
	return i;
}

void readbyte(unsigned char *pw0, unsigned char *pw1) {
uint8_t d0=0,d1=0;
pullup_off();

for(uint8_t i=0, r=1; i<8; i++){
	uint8_t p=readbit();
	if ( p & pwire0 ) d0|=r;
	if ( p & pwire1 ) d1|=r;
	r<<=1;	
	}
PORT_1wire|=out_1wire;
DDR_1wire|=out_1wire;
*pw0=d0;
*pw1=d1;
}

void writebit0(void){
	cli();
	DDR_1wire|=out_1wire;
	_delay_us(60);
	DDR_1wire&=~out_1wire;
	sei();
}

void writebit1(void){
	cli();
	DDR_1wire|=out_1wire;
	_delay_us(3);
	DDR_1wire&=~out_1wire;
	sei();
	_delay_us(58);
}

void writebyte(unsigned char byte){
unsigned char i;
	pullup_off();
	for(i=0; i<8; i++){
		if (byte&BIT(0))
			writebit1();
			else
			writebit0();
		byte>>=1;
		}
	PORT_1wire|=out_1wire;
	DDR_1wire |=out_1wire;
}

void detectPresence(void){
unsigned char i;
	pullup_off();
	out_1wire=pwire0|pwire1;
	
	DDR_1wire|=pwire0;
	DDR_1wire|=pwire1;
	_delay_ms(0.48);
	DDR_1wire&=~pwire0;
	DDR_1wire&=~pwire1;
	
	cli();
	_delay_us(60);
	i=PIN_1wire;
	sei();
	
	if ( i & pwire0 ) {out_1wire&=~pwire0; PORTDRIVE&=~DRIVE_TEN_1; PORTDRIVE&=~DRIVE_TEN_2;}
	if ( i & pwire1 ) {out_1wire&=~pwire1; }
	
	if ( out_1wire ){
		_delay_ms(0.42);
		i=PIN_1wire;
		if (!( i & pwire0 )) {out_1wire&=~pwire0; PORTDRIVE&=~DRIVE_TEN_1; PORTDRIVE&=~DRIVE_TEN_2;}
		if (!( i & pwire1 )) {out_1wire&=~pwire1; }
		}
	if (out_1wire==0) state_wire=0;
	else{
	PORT_1wire|=out_1wire;
	DDR_1wire|=out_1wire;
	}
}

//***************************
void scratchpad_save(void){
	detectPresence();
	if (out_1wire){
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0x4E);//WRITE SCRATCHPAD [4Eh]
		writebyte(0);//	TH
		writebyte(0);//	TL
		writebyte(0x7F);//	20 Statys 10bit //writebyte(0x60);//	Statys 12bit byte 4 Configuration Register* 0x7F - 12 bit 750mSek; 1F-9bit 94msek
		detectPresence();
		if (out_1wire){// EEPROM
			writebyte(0xCC);//SKIP ROM [CCh]
			writebyte(0x48);//COPY SCRATCHPAD [48h]
			
			PORT_1wire|=out_1wire;
			DDR_1wire|=out_1wire;
			}
		}
}
//*******end***1wire************

//**************************************************
void format_t(unsigned char p){
int t_point;
	t_point=((unsigned int)(tempir[p*2+1]<<8))|tempir[p*2];
	Temperatura[p]=t_point*10>>4;
}	

void receive_t(void){

	if ( state_wire==0 )
	{
	state_wire=1;
	detectPresence();
	return;
	}
	
	if ( state_wire==1 )
	{
	state_wire=2;
	writebyte(0xCC);//SKIP ROM [CCh]
	writebyte(0x44);//CONVERT T [44h]
	time_wire=100;  // time= time_wire/100
	return;
	}
	
	if ( state_wire==2 )
	{
	if (!time_wire) state_wire=3;
	return;
	}

	if ( state_wire==3 )
	{
	state_wire=4;
	detectPresence();
	return;
	}
	
	if ( state_wire==4 )
	{
	state_wire=0;
	writebyte(0xCC);//SKIP ROM [CCh]
	writebyte(0xBE);//READ SCRATCHPAD [BEh]

	readbyte(&tempir[0],&tempir[2]);
	readbyte(&tempir[1],&tempir[3]);
	
	format_t(0);
	format_t(1);
	
	if ( eep_error ) {PORTDRIVE &= ~DRIVE_TEN_1; PORTDRIVE &= ~DRIVE_TEN_2; return;}

//---------------------------------------------------------------	
/*	P.oF t_stat[0] -  ,      PD5 .    35  
	P.on t_stat[1] -  ,      PD5   37.*/
	if (Temperatura[ds0] <= t_stat[0]) PORTDRIVE |= DRIVE_TEN_1;
		else
		if (Temperatura[ds0] >= t_stat[1]) PORTDRIVE &= ~DRIVE_TEN_1;
/*
	   2 ()   PD6 , 
	      1 () */  
	if (Temperatura[ds0] <= t_stat[0]) PORTDRIVE &= ~DRIVE_TEN_2;
		else
		if (Temperatura[ds0] >= t_stat[1]) PORTDRIVE |= DRIVE_TEN_2;
//---------------------------------------------------------------	
	return;
	}
}

//**************************************
void result_in_bcd(int result)
{
    unsigned char dig_out=0;
	bcd_num1=0;
	bcd_num2=0;
	bcd_num3=0;
	bcd_num4=0;

	while (result >= 1000) {
	 result -= 1000;
 	 dig_out ++;
	}
	bcd_num1=dig_out;
    dig_out = 0;
	while (result >= 100)  {
 	 result -= 100;
	 dig_out ++;
	 }
	bcd_num2=dig_out;
	dig_out = 0;
	while (result >= 10)   {
	 result -= 10;
	 dig_out ++;
	 }
	bcd_num3=dig_out;
	bcd_num4=result;
}
//**************************************
void t_in_bcd(signed int t, unsigned char pin_wire){
unsigned char znak=0;

	 if ( !(pin_wire & out_1wire) ){
	 	lcd_buffer[0]	=font[f_seg_g];
		lcd_buffer[1]	=font[f_seg_g];
		lcd_buffer[2]	=font[f_seg_g];
		lcd_buffer[3]	=font[f_seg_g];
		}
		else
		{
		if ( t<0 ){
			znak=1;
			result_in_bcd( t * -1 );
			}
		else 
			result_in_bcd( t );
		
		if ( bcd_num1 ){  
			lcd_buffer[0]=font[bcd_num1];
			lcd_buffer[1]=font[bcd_num2];
			}
		else
		if ( bcd_num2 ){ 
			lcd_buffer[1]=font[bcd_num2]; 
			if ( znak ) lcd_buffer[0]=font[f_seg_g];
			}
			else
			if ( znak ) lcd_buffer[1]=font[f_seg_g];
		lcd_buffer[2]=font[bcd_num3]|font[tochka]; 
		lcd_buffer[3]=font[bcd_num4]; 
		}
}

//-------------------------------------
void skan_key(void)
{
unsigned char pin=0;
	key=0;
	//PORTZN &= ~BITZN;
	#ifdef katod
	PORTZN |= BITZN;
	#else
	PORTZN &= ~BITZN;
	#endif
	DDRLED=BIT(7);
	PORTLED=(unsigned char)~BIT(7);
	_delay_us(30);
	pin=PINLED|BIT(7);
	#ifdef katod
	PORTLED=0;
	#else
	PORTLED=0xFF;
	#endif
	if ( pin!= 0xFF ){
		time_view_zadanie=700;//0.05
		if (++anti_dr==10)
			{
			key= ~pin;
			}
		if (anti_dr==btn_speed){
			anti_dr=0;
			if (++n_btn_sp==4) btn_speed=15;
			if (  n_btn_sp==45) increment=10;
			}
		}
		else{
		btn_speed=200;
		n_btn_sp=0;
		anti_dr=0;
		increment=1;
		}
	DDRLED=0xFF;
}

//-------------------------------------
void display(void)
{
#ifdef katod
	PORTZN |= BITZN;
	
	if ( znmesto==0 )	PORTZN &= ~ZN0;
	else
	if ( znmesto==1 )	PORTZN &= ~ZN1;
	else
	if ( znmesto==2 )	PORTZN &= ~ZN2;
	else
	if ( znmesto==3 )	PORTZN &= ~ZN3;
	PORTLED=lcd_buffer[znmesto];
#else
	PORTZN &= ~BITZN;

	if ( znmesto==0 )	PORTZN |= ZN0;
	else
	if ( znmesto==1 )	PORTZN |= ZN1;
	else
	if ( znmesto==2 )	PORTZN |= ZN2;
	else
	if ( znmesto==3 )	PORTZN |= ZN3;
	PORTLED=~lcd_buffer[znmesto];
#endif
}
//-------------------------------------

void key_action(void)
{

if (key){
	if (view_zadanie)
	{
	if ( key & btn_plus ){
		if ( time_flag & BIT(_viewp) ) {time_flag &= ~BIT(_viewp); return;}
		if (t_stat[view_zadanie-1]<990) t_stat[view_zadanie-1]+=increment; else t_stat[view_zadanie-1]=0;
		//time[_viewp]=5;
		}
	if ( key & btn_minus ){
		if ( time_flag & BIT(_viewp) ) {time_flag &= ~BIT(_viewp); return;}
		if (t_stat[view_zadanie-1] > 0) t_stat[view_zadanie-1]-=increment; else t_stat[view_zadanie-1]=990;
		//time[_viewp]=5;
		}
	}
	if ( key & btn_set )
		{
		if ( ++view_zadanie==sizeof(t_stat)/2+1 ){
			view_zadanie=0;
			eep_error=0;
			unsigned char *p, crc=0;
			p=(unsigned char *)t_stat;
			for (unsigned char i=0, a=ADR_EEP; i<sizeof(t_stat); i++, a++, p++){
				EEPROM_write(a, *p );
				crc+=*p;
				}
			EEPROM_write( ADR_EEP+sizeof(t_stat), crc );
			}
		
		time_flag |= BIT(_viewp);
		time[_viewp]=100;
		}
	}
}


//-------------------------------------
void data_led(void)
{
unsigned char i;

for(i=0;i<4;i++)
	lcd_buffer[i]=0x0;
	
if (view_zadanie)
	{
	if ( time_flag & BIT(_viewp) )
		{
		if ( view_zadanie==1 ){
			lcd_buffer[1]=font[f_o];
			lcd_buffer[2]=font[f_n];
			}
			else{
			lcd_buffer[1]=font[0];
			lcd_buffer[2]=font[f_F];
			lcd_buffer[3]=font[f_F];
			}
		}
		else 
		t_in_bcd(t_stat[view_zadanie-1],pwire0);
		
	}
	else
	{
	if (eep_error)
		{
		lcd_buffer[1]=font[f_e];
		lcd_buffer[2]=font[f_e];
		lcd_buffer[3]=font[f_P];
		}
	else
	if ( PINDRIVE & swt_t2 ) 
		t_in_bcd(Temperatura[ds0],pwire0);
		else
		t_in_bcd(Temperatura[ds1],pwire1);
	}

}
//-------------------------------------

void data_eeprom_read(void){
unsigned char *p, crc=0;
	p=(unsigned char *)t_stat;
	for (unsigned char i=0, a=ADR_EEP; i<sizeof(t_stat); i++, a++, p++){
		*p=EEPROM_read(a);
		crc+=*p;
		}
	if ( crc != EEPROM_read(ADR_EEP+sizeof(t_stat)) ) eep_error=1;
}

int main(void)
{
	cli();
	avr_init();
	
	data_eeprom_read();

	detectPresence();
	if (out_1wire){
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0xBE);//READ SCRATCHPAD [BEh]
		readbyte(&tempir[0],&tempir[0]);//LSB
		readbyte(&tempir[0],&tempir[0]);//MSB
		readbyte(&tempir[0],&tempir[0]);
		readbyte(&tempir[0],&tempir[0]);
		readbyte(&tempir[0],&tempir[1]);
		//byte 4 Configuration Register* 0x1F-9bit 94msek; 3F-10bit 188msek; 0x5F-11bit 375msek; 0x7F - 12 bit 750mSek;   		
		if ( 	((tempir[0]!=0x7F)&&(pwire0 & out_1wire)) 
			||	((tempir[1]!=0x7F)&&(pwire1 & out_1wire)) ){
			scratchpad_save();//10ms
			_delay_ms(10);
			}
		tempir[0]=tempir[1]=0;
		}
	
while(1)
	{
	if ( time_flag & BIT(_led) )// 
		{
		time_flag &= ~BIT(_led);
		time[_led]=70;//~0.00
		
		skan_key();
		key_action();
		display();
		if ( ++znmesto==4 )
			{
			znmesto=0;
			data_led();
			}
		receive_t();
		}
	if ( time_flag & BIT(_01sek) )
		{
		time_flag &= ~BIT(_01sek);
		time[_01sek]=156;//~0,01
		time_wire--;
		if (--time_view_zadanie==0) view_zadanie=0;
		if (--time[_viewp]==0) time_flag &= ~BIT(_viewp);
		}
	}

}

